#include <Windows.h>
#include "lsapi.h"
#include <strsafe.h>
#include "ufmod.h"

// Constants
const char g_rcsRevision[]		= "1.0";
const char g_szAppName[]		= "LSXMPlayer";
const char g_szMsgHandler[]		= "LSXMPlayerManager";
const char g_szAuthor[]			= "Alurcard2";
const DWORD g_dwStyle			= WS_VISIBLE | WS_POPUP;
const DWORD g_dwExStyle			= WS_EX_NOACTIVATE | WS_EX_TOOLWINDOW;

const int g_lsMessages[] = {LM_GETREVID, 0};

// Function declarations
extern "C" __declspec( dllexport ) int initModuleEx(HWND hwndParent, HINSTANCE hDllInstance, LPCSTR szPath);
extern "C" __declspec( dllexport ) void quitModule(HINSTANCE hDllInstance);
bool CreateMessageHandlers(HINSTANCE hInst);
LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void BangPauseXM(HWND hwndCaller, LPCSTR pszArgs);
void BangPlayXM(HWND hwndCaller, LPCSTR pszArgs);
void BangPlayXMNoLoop(HWND hwndCaller, LPCSTR pszArgs);
void BangResumeXM(HWND hwndCaller, LPCSTR pszArgs);
void BangRestartXM(HWND hwndCalled, LPCSTR pszArgs);
void BangSetXMVolume(HWND hwndCaller, LPCSTR pszArgs);
void BangStopXM(HWND hwndCaller, LPCSTR pszArgs);
void BangXMGoto(HWND hwndCaller, LPCSTR pszArgs);

// Variables
HWND g_hwndMessageHandler = NULL;

// Implementation
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
int initModuleEx(HWND, HINSTANCE hDllInstance, LPCSTR)
{
	// Create window classes and the main window
	if (!CreateMessageHandlers(hDllInstance))
	{
		return 1;
	}

	// Register bangs
	AddBangCommand("!PauseXM", BangPauseXM);
	AddBangCommand("!PlayXM", BangPlayXM);
	AddBangCommand("!PlayXMNoLoop", BangPlayXMNoLoop);
	//AddBangCommand("!RestartXM", BangRestartXM);
	AddBangCommand("!ResumeXM", BangResumeXM);
	AddBangCommand("!SetXMVolume", BangSetXMVolume);
	AddBangCommand("!StopXM", BangStopXM);
	//AddBangCommand("!XMGoto", BangXMGoto);
	
	uFMOD_SetVolume(GetRCInt("XMPlayerVolume", 25));

	return 0; // Initialized succesfully
}

void quitModule(HINSTANCE hDllInstance)
{
	// Unregister bangs
	RemoveBangCommand("!PauseXM");
	RemoveBangCommand("!PlayXM");
	RemoveBangCommand("!PlayXMNoLoop");
	//RemoveBangCommand("!RestartXM");
	RemoveBangCommand("!ResumeXM");
	RemoveBangCommand("!SetXMVolume");
	RemoveBangCommand("!StopXM");
	//RemoveBangCommand("!XMGoto");

	// Stop any playing XM song
	uFMOD_StopSong();
	
	if (g_hwndMessageHandler)
	{
		// Unregister for LiteStep messages
		SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM)g_hwndMessageHandler, (LPARAM)g_lsMessages);

		// Destroy message handling windows
		DestroyWindow(g_hwndMessageHandler);

		g_hwndMessageHandler = NULL;
	}

	// Unregister window classes
	UnregisterClass(g_szMsgHandler, hDllInstance);
}

void BangPlayXM(HWND, LPCSTR pszArgs)
{
	uFMOD_PlaySong((void*)pszArgs, NULL, XM_FILE);
}

void BangPlayXMNoLoop(HWND, LPCSTR pszArgs)
{
	uFMOD_PlaySong((void*)pszArgs, NULL, XM_FILE | XM_NOLOOP);
}


void BangPauseXM(HWND, LPCSTR)
{
	uFMOD_Pause();
	Sleep(0);
}

void BangResumeXM(HWND, LPCSTR)
{
	uFMOD_Resume();
	Sleep(0);
}

void BangRestartXM(HWND, LPCSTR)
{
	uFMOD_Rewind();
}

void BangSetXMVolume(HWND, LPCSTR pszArgs)
{
	uFMOD_SetVolume(atoi(pszArgs));
}

void BangStopXM(HWND, LPCSTR)
{
	uFMOD_StopSong();
}

void BangXMGoto(HWND, LPCSTR pszArgs)
{
	uFMOD_Jump2Pattern(atoi(pszArgs));
}

bool CreateMessageHandlers(HINSTANCE hInst)
{
	WNDCLASSEX wc = {0};
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_NOCLOSE;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.hInstance = hInst;
	wc.lpszClassName = g_szMsgHandler;
	wc.hIconSm = 0;

	if (!RegisterClassEx(&wc))
		return false; // Failed to register message handler window class

	g_hwndMessageHandler = CreateWindowEx(WS_EX_TOOLWINDOW, g_szMsgHandler, 0, WS_POPUP, 0, 0, 0, 0, 0, 0, hInst, 0);

	if (!g_hwndMessageHandler)
		return false; // Failed to create message handler window

	// Register with LiteStep to receive LM_ messages
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)g_hwndMessageHandler, (LPARAM)g_lsMessages);

	return true;
}

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case LM_GETREVID:
		{
			size_t uLength;
			StringCchPrintf((char*)lParam, 64, "%s: %s", g_szAppName, g_rcsRevision);
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;
			lParam = NULL;
			return 0;
		}
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}